#include "VoxelSurface.h"
#include "utils.h"
#include <iostream>


using namespace std;



VoxelSurface::VoxelSurface(const Volume<float>& vol, float threshold)
{
  width  = vol.getWidth();
  height = vol.getHeight();
  depth  = vol.getDepth();  

  scalar_min = 1.0e6;
  scalar_max = -1.0e6;

  for(int i=1;i<depth-1;i++)					//1. Scan volume and select voxels on surface of value 'threshold'
    for(int j=1;j<height-1;j++)
      for(int k=1;k<width-1;k++) 
      {	
		coord3 c(k,j,i);								
		float tval =  vol(c);					//
		if (tval>=threshold)					//Found a voxel 'inside' the surface	
		{										//Now, to see if it's ON the surface, check that it has at least one
			bool select = false;				//neighbor which is 'outside' the surface
			for(int m=-1;m<=1;m++)
			  for(int n=-1;n<=1;n++)
				for(int p=-1;p<=1;p++) 
				  if(vol(k+p,j+n,i+m)<threshold) 
				  {
					select = true;
					goto BREAK;
				  }	     
			
			BREAK: 
		
			if (select)							//Voxel is on the surface: Add it to the set of surface-points,
			{									//together with its scalar value. Also update the scalar-range.
			  float value = tval;
		      points.push_back(Point3d(k,j,i,value));
			  scalar_min = min(scalar_min,value);		
			  scalar_max = max(scalar_max,value);		
			} 
		}
	  }
		    
   float norm[3],u1[3],u2[3], *nrm;
   const int NSZ = 2;
   vector<Point3d> nbs; nbs.reserve((NSZ+1)*(NSZ+1)*(NSZ+1));	  
   int NT = points.size();

   normals.resize(NT);							//2. Estimate surface normals for the selected surface points
   for(int pid=0;pid<NT;++pid)
   {
      const Point3d& p = points[pid];	  
  	  int i = p.z, j=p.y, k = p.x;

	  nbs.clear();								//Estimate surface normal by PCA on point's neighborhood
	  for(int m=-NSZ;m<=NSZ;m++)				//The normal is the smallest eigenvector
	    for(int n=-NSZ;n<=NSZ;n++)
		  for(int p=-NSZ;p<=NSZ;p++) 
		  {
		    bool inside = vol(k+p,j+n,i+m);
		    if (inside) nbs.push_back(Point3d(k+p,j+n,i+m)); 
		  }

	  if (nbs.size()>=3)						//Estimate plane only if there are enough neighbors...
	  {
	    float cl = plane(nbs,nrm=norm,u1,u2,0);
	    if (cl<0.2) nrm = 0;					//Anisotropy is planar: use plane normal, else fall back on default-normal
	  }
	  else nrm = 0;

      normals[pid] = (nrm)? Point3d(nrm) : Point3d();	
	}
}


void VoxelSurface::cut(const Volume<float>& volume_to_cut)
{
    scalar_min = 1.0e6;
	scalar_max = -1.0e6;

	for(int i=0;i<points.size();++i)
	{
		Point3d& p = points[i];
		coord3 c(p.x,p.y,p.z);
		
		float value = volume_to_cut(c);
		
		p.value = value;
 	    scalar_min = min(scalar_min,value);		
		scalar_max = max(scalar_max,value);		
    }
}

		
