#pragma once

#include <GLUT/glut.h>
#include "Point3d.h"
#include "Volume.h"


class VoxelSurface;


class VoxelSurfaceRenderer
{
public:

enum DRAW_MODE				//Style of voxel surface drawing:	
{
   DRAW_VOXELS =0,			//-use voxel cubes (slow, but shows the exact voxels)
   DRAW_SPLATS,				//-use shaded (oriented) points (fast, quite accurate)
   DRAW_GEOM_BALLS,			//-use geometric spheres (slow,inaccurate)
   DRAW_SPRITE_BALLS		//-use shaders (very fast and very accurate)
};     

						VoxelSurfaceRenderer();
void					draw(const VoxelSurface&);

float					light_diffuse;
float					light_specular;
float					specular_power;
float					light_pos[4];
int						use_mipmaps;									//use texture mipmaps for splatting or not
int						use_floating_textures;							//whether to use floating-point or 24-bit-fixed-point ones for the ball height
DRAW_MODE				draw_mode;										//how to draw the voxel surface
bool					use_colormap;

private:

void					initGL();													
void					makeTextures(int sampling_mode=-1);				//make all splat textures using some sampling mode
void					createSplatFloat(int size,GLfloat* img,int level);
void					createSplatRGBA(int size,GLubyte* img,int level);
void					createDisk(int size);
int						getScalingMode() const;							//return texture scaling mode depending on usage of mipmaps, etc
void					drawBalls(const VoxelSurface& surface,bool use_constant_radius);			
void					drawCubes(const VoxelSurface& surface);			
void					drawSplats(const VoxelSurface& surface);

float					ball_dt_offset;
float					splat_dt_offset;
float                   ball_size;										//ball size (in obj normalized coords) for the constant-radius ball drawing
int						sphere_faces;									//resolution of the GLUT spheres

GLuint					balltex[2];										//texture IDs for the ball splat (fixed-point and floating-point modes)
GLuint					ball_shader[3];									//shader IDs for the splat fragments
GLuint					ball_prog;										//shader program ID
int						splat_size;										//size (pow 2) of the splat texture
int						scaling_mode;									//internal texmapping param for splats
};






